#!/bin/ksh
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# Restricted Materials of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2005,2008 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

# @(#) 1.14 src/ppe/poe/install.linux/bin/pe_install.sh, ppe.poe.install, ppe_rhya, rhya0837c 08/09/16 10:30:42
# IBM PE install script.

function get_OS_version {
   if [[ -a /etc/SuSE-release ]] then
      OS_VERSION=`cat /etc/SuSE-release | grep "SUSE LINUX Enterprise Server 9"`
      if [[ -n $OS_VERSION ]] then
         CURR_OS=sles900
      else
         OS_VERSION=`cat /etc/SuSE-release | grep "SUSE Linux Enterprise Server 10"`
         if [[ -n $OS_VERSION ]] then
            CURR_OS=sles1000
         fi
      fi
   elif [[ -a /etc/redhat-release ]] then
      OS_VERSION=`cat /etc/redhat-release | grep "Red Hat Enterprise Linux" | grep "release 4"`
      if [[ -n ${OS_VERSION} ]] then
         CURR_OS=rh400
      else
         OS_VERSION=`cat /etc/redhat-release | grep "Red Hat Enterprise Linux" | grep "release 5"`
         if [[ -n $OS_VERSION ]] then
            CURR_OS=rh500
         fi
      fi
   fi
}

CURR_LIC_VER=""
CURR_PE_VER=""
CURR_LAPI_VER=""
HAVE_VERSION='N'
FULL_INSTALL='Y'

USAGE="${0}:\n
This script allows you to perform new installs, upgrade installs, or\n
installation of fixes for various RPM packages that comprise the\n
IBM Parallel Environment for Linux. This script can be used in either\n
interactive mode or in batch (or script) mode. While interactive mode\n
is meant for local installations, using this script in batch mode allows\n
you to perform remote installations when used in conjunction with\n
utilities such as 'dsh', 'rsh', and 'ssh'. You should be a root-level\n
user in order to run this script.\n
When used in interactive mode, this script will prompt for all\n
installation options. When used in batch mode, there are command line\n
options that correspond to all of the interactive mode prompts EXCEPT the\n
\t         Do full install of IBM Parallel Environment for Linux\n
prompt. This is because it is assumed that you have reviewed the terms\n
of the PE License Agreement and accept them. It is recommended that you\n
review the license agreement and manually accept it at least one time,\n
by running this script in interactive mode, and answering 'y' to the\n
following prompt:\n
\t         Do you want to review PE license agreement\n
You can then go ahead and perform further installs on multiple hosts\n
by running in batch mode, and the install script will automatically\n
do the full install, accept the license agreement, and put the required\n
license file on each host.\n
The command line options for this script are:\n
[-a]: Run this script interactively if this option is specified alone.\n
\t    If specified with a value of 'n', then this script will be run in\n
\t    batch mode.\n
[-h]: Print this help message.\n\n
The following are additional options that can be specified when running\n
in batch mode, and are specified when invoking this script. Each of these\n
options have a default value that is used if the option is not specified.\n
[-dir <path>]:  The full path where the PE RPM packages are located. The\n
\t            default path is the current working directory.\n
[-install_op {i | U}]:  The desired RPM installation operation. The\n
\t            acceptable options are 'i' for a new installation, and 'U',\n
\t            for an upgrade install. Both values correspond to the\n
\t            installation options for the rpm command. If -install_op is\n
\t            not specified, the installation operation defaults to a new\n
\t            install (as if you had specified 'i'). Each option performs\n
\t            the requested type of installation on the FULL set of PE RPMs.\n
\t            To perform a FIX upgrade to a particular PE RPM, specify this\n
\t            option with 'U', along with the -fix_level option.\n
[-fix_level <comp_name-ver.rel.mod.fix-build>]:  Specifies the fix package\n
\t            to install by component name and FIX level. The   comp_name\n
\t            portion of this string is either 'pe', or 'lapi'. The FIX\n
\t            level portion is specified in the format   ver.rel.mod.fix-build\n
\t            (for example, 4.2.0.1-0610a). Specifying this option without\n
\t            also using the -install_op option implies using -install_op\n
\t            with the 'U' install operation value.\n
[-ip_only {y | n}]:  Specifies whether to install only RPM packages for IP\n
\t            support, or the packages for User Space support also. Specifying\n
\t            a value of 'y' results in the installation of only the RPMs that\n
\t            support IP. A value of 'n' results in RPMs that support User\n
\t            Space being installed as well. If this option is not specified,\n
\t            the default behavior is to only install the RPMs that support IP.\n
\n
\tSee the PE Installation documentation for more information.\n"

if [[ $1 == '-h' ]] then
   print $USAGE
   exit 0
fi

# Determine Linux version, do it here as a check on the system
CURR_OS=""

get_OS_version

if [[ -z $CURR_OS ]] then
   print "Cannot determine version of operating system"
   exit 1
fi

PWD=`pwd`

# Query PE_LICENSE RPM
HAVE_PE_LICENSE='N'
QUERY_PE_LICENSE=`/bin/rpm -qa | grep IBM_pe_license`
if [[ -n $QUERY_PE_LICENSE && -d /etc/opt/ibmhpc/license ]] then
   HAVE_PE_LICENSE='Y'
fi

if [[ $# == 1 && $1 == '-a' ]] then
# Interactively prompt for install options

   # If PE license has not been installed, ask whether user wants to
   # do full or license only install
   if [[ $HAVE_PE_LICENSE == 'N' ]] then
      print "Do a full installation of IBM Parallel Environment for Linux?"
      print "Enter 'y' to install all PE and LAPI RPMs (default),"
      print -n "or 'n' to install just the PE License RPM:   "
      read all_comp
      if [[ $all_comp == 'Y' || $all_comp == 'y' || $all_comp == '' ]] then
         IBM_PE_LICENSE_ONLY=N
      else
         if [[ $all_comp == 'N' || $all_comp == 'n' ]] then
            IBM_PE_LICENSE_ONLY=Y
         else
            print "Abort: Illegal value.  Run ${0} -h to see usage."
            exit 1
         fi
      fi

      # To review license agreement or not.  Default is to review.
      print " "
      print " "
      print "Do you want to review the PE License Agreement and manually register your"
      print "acceptance of the terms, or just have your acceptance automatically registered"
      print "for you without reviewing the license agreement?"
      print -n "Enter 'y' to review it (default), or 'n' to automatically accept the terms:   "
      read review_lic
      if [[ $review_lic == 'Y' || $review_lic == 'y' || $review_lic == '' ]] then
         IBM_PE_LICENSE_PROMPT=Y
      else
         if [[ $review_lic == 'N' || $review_lic == 'n' ]] then
            IBM_PE_LICENSE_PROMPT=N
         else
            print "Abort: Illegal value.  Run ${0} -h to see usage."
            exit 1
         fi
      fi
   else
      # License has already been installed.  Go install full product.
      # This can happen when installing PTF or fixes
      print " "
      print " "
      print "The IBM PE license RPM has already been installed."
      print "Preparing to install/upgrade the IBM Parallel Environment product RPMs."
      IBM_PE_LICENSE_ONLY=N
      IBM_PE_LICENSE_PROMPT=N
   fi

   if [[ $IBM_PE_LICENSE_ONLY == 'N' ]] then
      print " "
      print " "
      print "Do you want to perform new RPM installations, update existing RPMs, or install"
      print "a fix to specific RPMs?"
      print "Enter 'i' for new installs (default), 'U' for"
      print -n "update installs, or 'f' for fix installs:   "
      read install_op
      if [[ $install_op == 'I' || $install_op == 'i' || $install_op == '' ]] then
         PE_INSTALL_OP='-i'
      elif [[ $install_op == 'u' || $install_op == 'U' ]] then
         PE_INSTALL_OP='-U'
      elif [[ $install_op == 'f' || $install_op == 'F' ]] then
         FULL_INSTALL='N'
         PE_INSTALL_OP='-U'
      else
         print "Abort: Illegal value.  Run ${0} -h to see usage".
         exit 1
      fi

      if [[ $FULL_INSTALL == 'Y' ]] then
# Note: set IBM_PE_IP_AND_US default to 'N' for GA, reset that to 'Y' for PTF
         print " "
         print " "
         print "Do you want to install both IP and User Space protocol support?"
         print -n "Enter 'y' for both IP & US support, or 'n' (default) for IP support only:   "
         read install_opt
         if [[ $install_opt == 'Y' || $install_opt == 'y' ]] then
            IBM_PE_IP_AND_US=Y
         else
            if [[ $install_opt == 'N' || $install_opt == 'n' || $install_opt == '' ]] then
               IBM_PE_IP_AND_US=N
            else
               print "Abort: Illegal value.  Run ${0} -h to see usage".
               exit 1
            fi
         fi
      else
         print " "
         print " "
         print "Please specify the component the fix is intended"
         print -n "for, by entering either 'pe' or 'lapi' (no default):   "
         read fix_component
         if [[ $fix_component == 'pe' || $fix_component == 'PE' ||
               $fix_component == 'lapi' || $fix_component == 'LAPI' ]] then
            print " "
            print " "
            print "Please specify the VRMF-level string from the name"
            print -n "of the file containing the fix (Example: 4.2.0.0-0610a):   "
            read fix_level
            if [[ $fix_component == 'pe' || $fix_component == 'PE' ]] then
               CURR_PE_VER=$fix_level
            else
               CURR_LAPI_VER=$fix_level
            fi
            HAVE_VERSION='Y'
         else
            print "Abort: Unknown component - ${fix_component}.  Run ${0} -h to see usage".
            exit 1
         fi
      fi
#  else
#  Note: There is no else clause because always use -i to install LAP licenses.
   fi

   # Current directory is default directory to look for IBM PE RPMs
   print " "
   print " "
   print "Please specify the directory where the RPM files are located."
   print "Hit the Enter key to use   ${PWD},"
   print -n "or enter the full path of the correct directory:  "
   read tmp_dir
   if [[ $tmp_dir == '' || $tmp_dir == ${PWD} ]] then
      IBM_PE_RPM_DIR=${PWD}
   else
      IBM_PE_RPM_DIR=${tmp_dir}
   fi

   print " "
   print " "
   print "Start IBM PE RPM installations now (last chance to abort)?"
   print "Hit the <Enter> key or enter 'y' to continue with the installs, or"
   print -n "enter any other character to exit this script without installing:   "
   read start_now
   if [[ $start_now != 'Y' && $start_now != 'y' && $start_now != '' ]] then
      print "Abort IBM PE installation as requested."
      exit 0
   fi
else
# Batch mode...
   IBM_PE_LICENSE_ONLY=N
   IBM_PE_LICENSE_PROMPT=N
# Note: set IBM_PE_IP_AND_US default to 'N' for GA, reset that to 'Y' for PTF
   IBM_PE_IP_AND_US=N
   IBM_PE_RPM_DIR=${PWD}
   PE_INSTALL_OP='-i'

# Parse install option from command line
   while [[ $1 = -* ]] do
      case $1 in
         -dir )
            shift
            if [[ $1 != -* ]] then
               IBM_PE_RPM_DIR=$1
            else
               print "ERROR: Missing value for '-dir' option."
               print "Run ${0} -h to see usage."
               exit 1
            fi
            ;;
         -fix_level )
            shift
            if [[ $1 != -* ]] then
               lpp_version=${1}
               if [[ -n `echo ${lpp_version} | grep "pe-"` ]] then
                  CURR_PE_VER=${lpp_version##"pe-"}
               elif [[ -n `echo ${lpp_version} | grep "lapi-"` ]] then
                  CURR_LAPI_VER=${lpp_version##"lapi-"}
               else
                  print "ERROR: Unknown component specified for '-fix_level' option."
                  print "Component must be either 'pe' or 'lapi'"
                  print "Run ${0} -h to see usage."
                  exit 1
               fi
               HAVE_VERSION='Y'
               FULL_INSTALL='N'
            else
               print "ERROR: Missing value for '-fix_level' option."
               print "Run ${0} -h to see usage."
               exit 1
            fi
            ;;

         -install_op )        # RPM operation (default is -i for install)
            shift
            if [[ $1 != -* ]] then
# If we want to support multiple install_ops, for example -i -f....
#              PE_INSTALL_OP="${PE_INSTALL_OP} -$1"
# Right now only allow -install_op to be specified ONCE
               if [[ HAVE_OP == 'Y' ]] then
                  print "ERROR: RPM install option has already been specified as $PE_INSTALL_OP"
                  exit 1
               fi 
               PE_INSTALL_OP="-$1"
               HAVE_OP='Y'
            else
               print "ERROR: Missing value for '-install_op' option."
               print "Run ${0} -h to see usage."
               exit 1
            fi
            ;;
         -ip_only )
            shift
            if [[ $1 != -* ]] then
               if [[ $1 == 'Y' || $1 == 'y' ]] then
                  IBM_PE_IP_AND_US='N'
               elif [[ $1 == 'N' || $1 == 'n' ]] then
                  IBM_PE_IP_AND_US='Y'
               else
                  print "ERROR: Value for '-ip_only' option must be either 'y' or 'n'"
                  print "Run ${0} -h to see usage."
                  exit 1
               fi
            else
               print "ERROR: Missing value for '-ip_only' option."
               print "Run ${0} -h to see usage."
               exit 1
            fi
            ;;
      esac
      shift
   done

# Set install op for FIX
   if [[ $FULL_INSTALL == 'N' && $PE_INSTALL_OP != '-U' ]] then
      PE_INSTALL_OP='-U'
   fi
fi

ARCH=`arch`

# Get the PE and LAPI Version
GOT_PE=0
GOT_LAPI=0

# We only put a license RPM on the GA CD and not on a PTF tar file.
# But we always specify the GA license RPM version in the PE_LAPI_VERSION
# file -- even for a PTF release.
# The reason: If a customer wants to install a PTF version on a brand
# new machine all he needs to do is to copy the license RPM from the
# GA CD and then use this install script to install the license plus
# the product PTF all in one shot. 
GOT_LIC=0

if [[ $HAVE_VERSION == 'N' ]] then
   if [[ -a ${IBM_PE_RPM_DIR}/PE_LAPI_VERSION ]] then
      if [[ $ARCH == 'ppc64' ]] then
         PREFIX="P:"
      else
         PREFIX="X:"
      fi
      exec 3< ${IBM_PE_RPM_DIR}/PE_LAPI_VERSION
      while read -u3 lpp_version ; do
          if [[ ${lpp_version} == "PE_Version:" ]] then
             while read -u3 lpp_version ; do
                if [[ ${lpp_version} != "End_PE_Version" ]] then
                   found_OS=`echo ${lpp_version} | grep ${PREFIX}${CURR_OS}`
                   if [[ -n $found_OS ]] then
                      CURR_PE_VER=${lpp_version##${PREFIX}}
                      GOT_PE=1
                      break
                   elif [[ -n `echo ${lpp_version} | grep "${PREFIX}LIC:"` ]] then
                      CURR_LIC_VER=${lpp_version##"${PREFIX}LIC:"}
                      GOT_LIC=1
                   fi
                else
                   print "Error: IBM PE does not support $CURR_OS"
                   exit 1
                fi
             done
          elif [[ ${lpp_version} == "LAPI_Version:" ]] then
             while read -u3 lpp_version ; do
                if [[ ${lpp_version} != "End_LAPI_Version" ]] then
                   found_OS=`echo ${lpp_version} | grep ${PREFIX}${CURR_OS}`
                   if [[ -n $found_OS ]] then
                      CURR_LAPI_VER=${lpp_version##${PREFIX}}
                      GOT_LAPI=1
                      break
                   fi
                else
                   print "Error: IBM LAPI does not support $CURR_OS"
                   exit 1
                fi
             done
          fi

          if [[ $GOT_LIC == 1 && $GOT_PE == 1 && $GOT_LAPI == 1 ]] then
             break
          fi
      done
   else
      print "Abort: file PE_LAPI_VERSION not found in ${IBM_PE_RPM_DIR}"
      exit 1
   fi
fi

# This test is mainly for internal testing to make sure that the
# PE_LAPI_VERSION has the correct information.
if [[ $GOT_LIC == 0 && HAVE_PE_LICENSE == 'N' ]] then
   print "Error: IBM PE License RPM version not found"
   exit 1
fi

# Debug
# print "IBM_PE_LICENSE_ONLY=$IBM_PE_LICENSE_ONLY"
# print "IBM_PE_LICENSE_PROMPT=$IBM_PE_LICENSE_PROMPT"
# print "IBM_PE_IP_AND_US=$IBM_PE_IP_AND_US"
# print "IBM_PE_RPM_DIR=$IBM_PE_RPM_DIR"
# print "IBM_PE_INSTALL_OP=$PE_INSTALL_OP"
# print "CURR_LIC_VER=${CURR_LIC_VER}"
# print "CURR_PE_VER=${CURR_PE_VER}"
# print "CURR_LAPI_VER=$CURR_LAPI_VER"
# exit

# Compose the RPM file names
if [[ $ARCH == 'ppc64' ]] then
   install64=Y
   LIC_RPM=${IBM_PE_RPM_DIR}/IBM_pe_license-${CURR_LIC_VER}.ppc.rpm
   PPE_BASE_RPM=${IBM_PE_RPM_DIR}/ppe_ppc_base_32bit_${CURR_PE_VER}.ppc.rpm
   PPE_64BIT_RPM=${IBM_PE_RPM_DIR}/ppe_ppc_64bit_${CURR_PE_VER}.ppc64.rpm
   LAPI_32BIT_BASE_IP_RPM=${IBM_PE_RPM_DIR}/lapi_ppc_32bit_base_IP_${CURR_LAPI_VER}.ppc.rpm
   LAPI_64BIT_IP_RPM=${IBM_PE_RPM_DIR}/lapi_ppc_64bit_IP_${CURR_LAPI_VER}.ppc64.rpm
   if [[ $IBM_PE_IP_AND_US == 'Y' ]] then
      LAPI_32BIT_US_RPM=${IBM_PE_RPM_DIR}/lapi_ppc_32bit_US_${CURR_LAPI_VER}.ppc.rpm
      LAPI_64BIT_US_RPM=${IBM_PE_RPM_DIR}/lapi_ppc_64bit_US_${CURR_LAPI_VER}.ppc64.rpm
   fi
else
   if [[ $ARCH == 'i386' || $ARCH == 'i686' ]] then
      install64=N
      LIC_RPM=${IBM_PE_RPM_DIR}/IBM_pe_license-${CURR_LIC_VER}.i386.rpm
      PPE_BASE_RPM=${IBM_PE_RPM_DIR}/ppe_x86_base_32bit_${CURR_PE_VER}.i386.rpm
      PPE_64BIT_RPM=''
      LAPI_32BIT_BASE_IP_RPM=${IBM_PE_RPM_DIR}/lapi_x86_32bit_base_IP_${CURR_LAPI_VER}.i386.rpm
      LAPI_64BIT_IP_RPM=''
      if [[ $IBM_PE_IP_AND_US == 'Y' ]] then
         LAPI_32BIT_US_RPM=${IBM_PE_RPM_DIR}/lapi_x86_32bit_US_${CURR_LAPI_VER}.i386.rpm
         LAPI_64BIT_US_RPM=''
      fi
   else
      if [[ $ARCH == 'x86_64' ]] then
         install64=Y
         LIC_RPM=${IBM_PE_RPM_DIR}/IBM_pe_license-${CURR_LIC_VER}.i386.rpm
         PPE_BASE_RPM=${IBM_PE_RPM_DIR}/ppe_x86_base_32bit_${CURR_PE_VER}.i386.rpm
         PPE_64BIT_RPM=${IBM_PE_RPM_DIR}/ppe_x86_64bit_${CURR_PE_VER}.x86_64.rpm
         LAPI_32BIT_BASE_IP_RPM=${IBM_PE_RPM_DIR}/lapi_x86_32bit_base_IP_${CURR_LAPI_VER}.i386.rpm
         LAPI_64BIT_IP_RPM=${IBM_PE_RPM_DIR}/lapi_x86_64bit_IP_${CURR_LAPI_VER}.x86_64.rpm
         if [[ $IBM_PE_IP_AND_US == 'Y' ]] then
            LAPI_32BIT_US_RPM=${IBM_PE_RPM_DIR}/lapi_x86_32bit_US_${CURR_LAPI_VER}.i386.rpm
            LAPI_64BIT_US_RPM=${IBM_PE_RPM_DIR}/lapi_x86_64bit_US_${CURR_LAPI_VER}.x86_64.rpm
         fi
      else
         print "Abort: Platform ...${ARCH}... is not supported."
         exit 1
      fi
   fi
fi

# Check RPM files existence...
#     If license has not been installed the license RPM must be there
#     Other RPM files: If install option is '-i' all RPM Files must present
#                      If install option is '-U' must have at least one RPM file
MISSING_FILES='N'
HAVE_AT_LEAST_ONE_RPM_FILE='N'

if [[ $HAVE_PE_LICENSE == 'N' && ! -a $LIC_RPM ]] then
   print "ERROR: IBM PE License RPM file -- ${LIC_RPM#${IBM_PE_RPM_DIR}/} was not found in ${IBM_PE_RPM_DIR}."
   MISSING_FILES='Y'
fi

if [[ $IBM_PE_LICENSE_ONLY == 'N' ]] then
   if [[ ! -a $PPE_BASE_RPM ]] then
      HAVE_PPE_BASE_RPM='N'
      if [[ $FULL_INSTALL == 'Y' ]] then
         print "ERROR: IBM PE 32bit Base RPM file -- ${PPE_BASE_RPM#${IBM_PE_RPM_DIR}/} was not found in ${IBM_PE_RPM_DIR}."
         MISSING_FILES='Y'
      fi
   else
      HAVE_PPE_BASE_RPM='Y'
      HAVE_AT_LEAST_ONE_RPM_FILE='Y'
   fi

   if [[ ! -a $LAPI_32BIT_BASE_IP_RPM ]] then
      HAVE_LAPI_32BIT_BASE_IP_RPM='N'
      if [[ $FULL_INSTALL == 'Y' ]] then
         print "ERROR: IBM LAPI 32bit Base IP RPM file-- ${LAPI_32BIT_BASE_IP_RPM#${IBM_PE_RPM_DIR}/} was not found in ${IBM_PE_RPM_DIR}."
         MISSING_FILES='Y'
      fi
   else
      HAVE_LAPI_32BIT_BASE_IP_RPM='Y'
      HAVE_AT_LEAST_ONE_RPM_FILE='Y'
   fi

   if [[ $IBM_PE_IP_AND_US == 'Y' && ! -a $LAPI_32BIT_US_RPM ]] then
      HAVE_LAPI_32BIT_US_RPM='N'
      if [[ $FULL_INSTALL == 'Y' ]] then
         print "ERROR: IBM LAPI 32bit US RPM file -- ${LAPI_32BIT_US_RPM#${IBM_PE_RPM_DIR}/} was not found in ${IBM_PE_RPM_DIR}."
         MISSING_FILES='Y'
      fi
   else
      HAVE_LAPI_32BIT_US_RPM='Y'
      HAVE_AT_LEAST_ONE_RPM_FILE='Y'
   fi

   # If customer can decide not to install 64BIT RPMs on 64BIT platforms
   if [[ $install64 == 'Y'  ]] then
      if [[ ! -a $PPE_64BIT_RPM ]] then
         HAVE_PPE_64BIT_RPM='N'
         if [[ $FULL_INSTALL == 'Y' ]] then
            print "ERROR: IBM PE 64bit RPM file -- ${PPE_64BIT_RPM#${IBM_PE_RPM_DIR}/} was not found in ${IBM_PE_RPM_DIR}."
            MISSING_FILES='Y'
         fi
      else
         HAVE_PPE_64BIT_RPM='Y'
         HAVE_AT_LEAST_ONE_RPM_FILE='Y'
      fi

      if [[ ! -a $LAPI_64BIT_IP_RPM ]] then
         HAVE_LAPI_64BIT_IP_RPM='N'
         if [[ $FULL_INSTALL == 'Y' ]] then
            print "ERROR: IBM LAPI 64bit IP RPM file -- ${LAPI_64BIT_IP_RPM#${IBM_PE_RPM_DIR}/} was not found in ${IBM_PE_RPM_DIR}."
            MISSING_FILES='Y'
         fi
      else
         HAVE_LAPI_64BIT_IP_RPM='Y'
         HAVE_AT_LEAST_ONE_RPM_FILE='Y'
      fi

      if [[ $IBM_PE_IP_AND_US == 'Y' && ! -a $LAPI_64BIT_US_RPM ]] then
         HAVE_LAPI_64BIT_US_RPM='N'
         if [[ $FULL_INSTALL == 'Y' ]] then
            print "ERROR: IBM LAPI 64bit US RPM file -- ${LAPI_64BIT_US_RPM#${IBM_PE_RPM_DIR}/} was not found in ${IBM_PE_RPM_DIR}."
            MISSING_FILES='Y'
         fi
      else
         HAVE_LAPI_64BIT_US_RPM='Y'
         HAVE_AT_LEAST_ONE_RPM_FILE='Y'
      fi
   fi
fi

if [[ $FULL_INSTALL == 'Y' ]] then
   if [[ $MISSING_FILES == 'Y' ]] then
      print "Abort: IBM PE full install has stopped, due to missing RPM file(s)."
      exit 1
   fi
fi

if [[ $PE_INSTALL_OP == '-U' && $HAVE_AT_LEAST_ONE_RPM_FILE == 'N' ]] then
   print "Abort: IBM PE install upgrade has stopped, due to missing RPM file(s)."
   exit 1
fi

# Start installing...install license first if it is not there
print "Start installing IBM Parallel Environment ...."
if [[ $HAVE_PE_LICENSE == 'N' ]] then
   export IBM_PE_LICENSE_PROMPT=${IBM_PE_LICENSE_PROMPT}
   print "rpm -i ${LIC_RPM}"
   rpm -i ${LIC_RPM}
   rc=$?
   if [[ $rc == 0 ]] then
      # Note: In silent mode license has already been silently accepted.
      if [[ $IBM_PE_LICENSE_PROMPT == 'Y' ]] then
         /opt/ibmhpc/install/sbin/accept_ppe_license.sh
         rc=$?
         if [[ $rc != 0 ]] then
            print "Abort: IBM PE license agreement has not been accepted."
            rpm -e IBM_pe_license
            exit $rc
         fi
      fi
      if [[ $IBM_PE_LICENSE_ONLY == 'Y' ]] then
         exit 0 
      fi
   else
      print "Abort: Error installing IBM PE license RPM (rc=$rc)."
      exit $rc
   fi
fi

# Add installing RSCT here for 0606 PTF ......

# Installing PE/LAPI components.  Have to check the HAVE_FILE
# variable because upgrade '-U' allow skipping files.
if [[ $HAVE_LAPI_32BIT_BASE_IP_RPM == 'Y' ]] then
   print "rpm ${PE_INSTALL_OP} ${LAPI_32BIT_BASE_IP_RPM}"
   rpm ${PE_INSTALL_OP} ${LAPI_32BIT_BASE_IP_RPM}
   rc=$?
   if [[ $rc == 0 ]] then
      print "Successfully installed IBM LAPI 32bit BASE IP RPM"
   else
      print "Abort: Failed to install ${LAPI_32BIT_BASE_IP_RPM} (rc=$rc)."
      exit $rc
   fi
fi

if [[ $IBM_PE_IP_AND_US == 'Y' && $HAVE_LAPI_32BIT_US_RPM == 'Y' ]] then
   print "rpm ${PE_INSTALL_OP} ${LAPI_32BIT_US_RPM}"
   rpm ${PE_INSTALL_OP} ${LAPI_32BIT_US_RPM}
   rc=$?
   if [[ $rc == 0 ]] then
      print "Successfully installed IBM LAPI 32bit US RPM"
   else
      print "Abort: Failed to install ${LAPI_32BIT_US_RPM} (rc=$rc)."
      exit $rc
   fi
fi

if [[ $HAVE_PPE_BASE_RPM == 'Y' ]] then
   print "rpm ${PE_INSTALL_OP} ${PPE_BASE_RPM}"
   rpm ${PE_INSTALL_OP} ${PPE_BASE_RPM}
   rc=$?
   if [[ $rc == 0 ]] then
      print "Successfully installed IBM PE 32bit BASE RPM"
   else
      print "Abort: Failed to install ${PPE_BASE_RPM} (rc=$rc)."
      exit $rc
   fi
fi

# If 64BIT RPM exist install IBM 64BIT PE RPM which will in turn
# install 64BIT LAPI RPM.
if [[ $install64 == 'Y' ]]  then
   if [[ $HAVE_LAPI_64BIT_IP_RPM == 'Y' ]] then
      print "rpm ${PE_INSTALL_OP} ${LAPI_64BIT_IP_RPM}"
      rpm ${PE_INSTALL_OP} ${LAPI_64BIT_IP_RPM}
      rc=$?
      if [[ $rc == 0 ]] then
         print "Successfully installed IBM LAPI 64bit IP RPM"
      else
         print "Abort: Failed to install ${LAPI_64BIT_IP_RPM} (rc=$rc)."
         exit $rc
      fi
   fi

   if [[ $IBM_PE_IP_AND_US == 'Y' && $HAVE_LAPI_64BIT_US_RPM == 'Y' ]] then
      print "rpm ${PE_INSTALL_OP} ${LAPI_64BIT_US_RPM}"
      rpm ${PE_INSTALL_OP} ${LAPI_64BIT_US_RPM}
      rc=$?
      if [[ $rc == 0 ]] then
         print "Successfully installed IBM LAPI 64bit US RPM"
      else
         print "Abort: Failed to install ${LAPI_64BIT_US_RPM} (rc=$rc)."
         exit $rc
      fi
   fi

   if [[ $HAVE_PPE_64BIT_RPM == 'Y' ]] then
      print "rpm ${PE_INSTALL_OP} ${PPE_64BIT_RPM}"
      rpm ${PE_INSTALL_OP} ${PPE_64BIT_RPM}
      rc=$?
      if [[ $rc == 0 ]] then
         print "Successfully installed IBM PE 64bit RPM"
      else
         print "Abort: Failed to install ${PPE_64BIT_RPM} (rc=$rc)."
         exit $rc
      fi
   fi
fi

print "Installation of IBM Parallel Environment completed."
exit 0
