/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2008               */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
#pragma comment(copyright,"5765-543 (C) Copyright IBM Corp. 2008 @(#) 1.2 src/ppe/hpct/source/examples/mp_profiler/cexamples/control.c, ppe.hpct, ppe_rhya, rhyas001a 08/11/04 11:40:17")
#include <mpt.h>
#include <mpi.h>
#include <mpi_trace_ids.h>


int MT_trace_event (int id) {
    struct MT_envstruct env;
    MT_get_environment(&env);
    printf("[task %d] %s (%d) \n",env.mpirank,MT_get_mpi_name(id),id);
    /* return 1 to trace, and 0 to omit this mpi call*/
    return 1;
}

int MT_output_trace(int rank) {


    /* if((rank<16)||_MT_trace_ev_(rank)==1) */
    if(rank<32)
        return 1;
    else
        return 0;
}


int MT_output_text(void) {

   struct MT_envstruct env;
   struct MT_summarystruct summary,stackusage,heapusage,funcname,avghops;

   int flat_prorfile=0;
   char *ptr, filename[80];
   FILE *fh;
   double total_comm, avg_bytes, hop_sum, *data_double, *hop;
   int i,k,id,rc,rc1;
   char sformat[] = "%-22s %12lld    %11.1f   %12.3f\n";
   int mpirank;

   double *stack_size,*heap_size;


   rc = PMPI_Comm_rank(MPI_COMM_WORLD, &mpirank);


   MT_get_environment(&env);

   MT_get_allresults(COMMUNICATIONTIME, ALLMPI_ID, &summary);

   if( (mpirank==0) || (mpirank==summary.min_rank) || (mpirank==summary.max_rank) || (mpirank==summary.med_rank)) {

   	sprintf(filename, "mpi_profile.%d", mpirank);
   	fh = fopen(filename, "w");
   	if (fh == NULL)
   	{
   	      printf("task %d failed to open the trace file %s\n", mpirank, filename);
   	      return -1;
   	}
   	fprintf(fh,"-----------------------------------------------------------------\n");
   	fprintf(fh,"MPI Routine                  #calls     avg. bytes      time(sec)\n");
   	fprintf(fh,"-----------------------------------------------------------------\n");


  	total_comm=0.0;
   	for (id=0; id< env.nmpi; id++)
   	{
   	  if (MT_get_mpi_counts(id) > 0LL)
   	  {
   	    avg_bytes = MT_get_mpi_bytes(id) / ((double) MT_get_mpi_counts(id));
   	    fprintf(fh,sformat, MT_get_mpi_name(id), MT_get_mpi_counts(id), avg_bytes, MT_get_mpi_time(id)); 
   	    total_comm+=MT_get_mpi_time(id);
   	  }
   	} 

   	fprintf(fh,"-----------------------------------------------------------------\n");
   	fprintf(fh,"total communication time = %.3f seconds.\n", total_comm);

   	fprintf(fh,"total elapsed time       = %.3f seconds.\n", MT_get_elapsed_time());


   	fprintf(fh,"\n");

   	MT_print_message_size_dist(fh);


	fclose(fh);

   }

    rc=MT_get_allresults(STACK, -1, &stackusage);
    MT_get_allresults(HEAP, -1, &heapusage);
    MT_get_allresults(MAXSTACKFUNC, -1, &funcname);
    rc1=MT_get_allresults(AVGHOPS, -1, &avghops); 


   if (mpirank==0) /* print the summary for all nodes */
   {


         sprintf(filename, "mpi_profile.%d", mpirank);
         fh = fopen(filename, "a");


         fprintf(fh,"-----------------------------------------------------------------\n"
);
         fprintf(fh, "\nCommunication summary for all tasks:\n");
         fprintf(fh, "\n");
         fprintf(fh, "  minimum communication time = %.3lf sec for task %d\n",
                 *(double *)summary.min_result, summary.min_rank);
         fprintf(fh, "  median  communication time = %.3lf sec for task %d\n",
                 *(double *)summary.med_result, summary.med_rank);
         fprintf(fh, "  maximum communication time = %.3lf sec for task %d\n",
                 *(double *)summary.max_result, summary.max_rank);
         fprintf(fh, "\n");

/*
         hop_sum=0.0;
	 for(i=0;i<env.ntasks;i++) hop_sum += MT_get_avg_hops(i);
*/


	 hop_sum = *(double *) avghops.sum_result;

	 data_double= (double *)summary.all_result;


         if(rc==1) { /* function trace enabled */
       	    fprintf(fh, "-----------------------------------------------------------------------\n");
            fprintf(fh, " Memory statisics: max_heap and max_stack in MBytes from each MPI task.\n");
            fprintf(fh, " The routine where the stack reached the maximum is also listed.\n");
            fprintf(fh, "-----------------------------------------------------------------------\n");
            fprintf(fh, "  task    max_heap(MB)   max_stack(MB)  routine\n");

            stack_size=(double *)stackusage.all_result;
            heap_size=(double *)heapusage.all_result;
            for (i=0; i<env.ntasks; i++)
            {
               ptr = funcname.all_result+i*sizeof(char)*80;
               fprintf(fh, "%6d  %10.2lf      %10.2lf      %s\n", i, heap_size[i], stack_size[i], ptr);
            }
         }


         fclose(fh);
    }

    return 1;


}
