/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2008               */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
#pragma comment(copyright,"5765-543 (C) Copyright IBM Corp. 2008 @(#) 1.2 src/ppe/hpct/source/examples/plugins/single.c, ppe.hpct, ppe_rhya 08/12/01 14:19:42")
/****************************************************************************
 *
 *   Returning only the results from PRINT_TASK
 *
 *               C. Pospiech ** ACTC
 *
 *  $Id: single.c,v 1.5 2008/02/05 11:43:32 cp Exp $
 *
 ****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <mpi.h>

#include "hpc_data.h"
#include "hpc_agg.h"

#define TRUE_ 1
#define FALSE_ 0

#define PRINT_TASK_DEFAULT 0


int distributor(void) {

  return(0);
}

int aggregator(int num_in, hpm_event_vector in,
               int *num_out, hpm_event_vector *out,
	       int *is_print_task) {

  int MPI_is_initialized, MPI_Init_done_here;
  int argc=1, print_task=PRINT_TASK_DEFAULT;
  char *dummy_string = "dummy String";
  char *dummy_vector[1] = {dummy_string};
  char **argv = dummy_vector;
  char *envstr = NULL;
  int my_process_id = MPI_TASK_ID_NOT_AVAILABE;

  /*
   * This plugin is returning only the results from one task.
   * This task is selected by setting PRINT_TASK
   */
  envstr = getenv("HPM_PRINT_TASK");
  if (envstr) print_task=strtol(envstr, NULL, 10);

  /* 
   * Initialize MPI
   * Depending on whether this function is called from hpmcount
   * or libhpm, MPI_Init might not be needed.
   * So we better check whether MPI has already been initialized.
   */
  if (MPI_Initialized( &MPI_is_initialized ) != MPI_SUCCESS) {
    printf("Error at MPI_Initialized\n");
    MPI_Abort(MPI_COMM_WORLD,4);
  }
  MPI_Init_done_here = !MPI_is_initialized;
  if (MPI_Init_done_here && (MPI_Init(&argc, &argv) != MPI_SUCCESS)) {
    printf("Error at MPI_Init\n");
    MPI_Abort(MPI_COMM_WORLD,4);
  }

  /* Get the MPI task id */
  if (MPI_Comm_rank(MPI_COMM_WORLD, &my_process_id) != MPI_SUCCESS) {
    printf("Error at MPI_Comm_rank\n");
    MPI_Abort(MPI_COMM_WORLD,4);
  }

  /* Generate the output data on the PRINT_TASK only */
  if (my_process_id == print_task) {

    /*
     * On the print task we do the same as mirror.so -
     * The input is mirrored to the output.
     */

    *num_out = num_in;
    *out = in;
    *is_print_task = TRUE_;
  } else { /* other tasks than the print task */
    *num_out = 0;
    *out = NULL;
    *is_print_task = FALSE_;
  }

  if (MPI_Init_done_here) MPI_Finalize();

  return(0);
}
