/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 1993,2008          */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
#ifndef PM_SSM_H
#define PM_SSM_H
/* "@(#) 1.37.2.25 src/ppe/poe/include/pm_SSM.h, ppe.poe.pm, ppe_rhya, rhyas002a 08/12/09 10:07:43\0" */
/****************************************************************************

 Name: pm_SSM.h

 Description: Prototypes, structure types and enumerated types for the
                          partition manager SSM functions.

 Licensed Materials - Property of IBM
 All rights reserved.
 US Government Users Restriced Rights -
 Use, duplication or disclosure restricted by
 GSA ADP Schedule Contract with IBM Corp.

 History:
   Date     Who Mod ID  Description
   ----     --- ------  -----------
   02/10/95 RAD M02     Feature 9963. POE support for Debugger Virtual Channel
                        by creating new pipes between pmd and the debug engine
                        to carry special SSM_DBE packets.
   03/15/95 PDD M03     Added new trace integration destination and message types
                        for VT support (feature 9968).
   04/07/95 rad M04     Added new functions to support copying of byte streems
                        from file descriptor to file descriptor to minimize the
                        usage of dynamically allocated storage.
   04/17/95 GJM M05     Added SSM_MPCI_CONNECT_DATA
   02/27/96 WJF M06     Added external debugger message types
   04/16/96 WJF M07     Added version message type
   05/03/96 NWN M08     Added DBE version message type
   05/10/96 WJF M09     Added CHILD_STOPPED and SEND_CHILD_STOPPED
   04/28/97 rad M10     Added SSM_DEBUG_EXIT for defect 21096.
   12/11/97 Adn M11     Add support for Checkpoint/Restart (F5055)
   06/28/98 Adn M12     Defect 41428 - Avoid deadlock in case a task fails to perform chkpt
   08/20/98 PFC M13     Defect 42600 - added new SSMs for MPI finalize synchronization.    
   11/17/98 PFC M14     Feature 15648 - added new SSM for SP Security (full DCE) exploitation
   02/12/99 WGT M15     Defect 47377 - change all long ints to simple ints for 64-bit
   06/23/99 PFC M16     Added SSM_BROADCAST for 8192 task support (Feature 46440).
   11/29/99 jhr         Added SSM_CMD_STR_COUNT to allow sending n SSM_CMD_STR requests
   12/01/99 PFC M17     Added new SSM for ident replacement function (Defect 52094)
                        and renumbered DCE creds SSM to ensure Troutbeck compatability
                        of ident replacement function SSM.
   09/14/00 PFC M18     Removed dbe/pedb/xpdbx related code (Defect 67545)
   04/10/00 GJM M19     Add support for Checkpoint/Restart (F63772)
   04/16/02 PFC M20     Add support for Federation multiple adapters & striping  (F79668)
   08/16/02 PFC M21     Add cluster based security (CTsec) support (F82715)
   10/23/02 PFC M22     Add additional security (CTsec) message (D87919)
   01/03/03 PFC M23     Add new UDP interface (D89076)
   09/19/05 GJM M24     Added SSM_MPI/LAPI_GLOBAL_INFO_BIN (D127544)
   11/08/05 PFC M25     Added Infiniband support (F129201)
   12/16/05 GJM M26    Added SSMs for local and global NTBL info (F130357)
   05/26/06 PFC M27     PE 4.3 removed VT & PSSP security SSMs (F135142)
   07/10/06 ECL M28     Add SSM_GETHOSTNAME (LI 2W9)
   06/04/08 PFC M29     PDBX removal (F150811)
   07/14/08 PFC M30     0P5 - 1024K task scaling phase 2 (F150649)
   08/01/08 ECL M31     0P5 - rename SSM_PROMPT to SSM_GO (25) (D152612)
   11/21/08 MYU M32     MP_SYNC_QP (D153128)

 ***************************************************************************/

/**************************************************************
  pm structured socket message prototypes
 **************************************************************/

/*--------------------------------------------------------------

  receive a structured socket message

  Parameters:     A0 = pm_SSM_read(A1, A2, A3, A4, A5, A6)

                  A1    socket to read data from      int
                  A2    pointer to buffer             char **
                  A3    length of buffer              int *
                  A4    enumerated message type       int *
                  A5    source node                   int *
                  A6    destination node              int *

                  A0    Return Code                   int
-----------------------------------------------------------------*/

int pm_SSM_read(int, char**, int*, int*, int*, int*);

/*--------------------------------------------------------------

  recv an ssm buffer

  Parameters:     A0 = pm_recv_buf(A1, A2, A3, A4)

                  A1    socket to read data from      int
                  A2    buffer to read data to        char *
                  A3    number of bytes to recv       int  M15
                  A4    actual bytes received         int * M15

                  A0    Return Code                   int
-----------------------------------------------------------------*/

int pm_recvSSM_buf(int, char*, int, int*);  /* M15 */

/*--------------------------------------------------------------

  send a structured socket message

  Parameters:     A0 = pm_SSM_write(A1, A2, A3, A4, A5, A6)

                  A1    socket ID                     int
                  A2    buffer                        char *
                  A3    size of buffer to send        int   M15
                  A4    message type                  int
                  A5    source node                   int
                  A6    destination node              int

                  A0    Return Code                   int
-----------------------------------------------------------------*/

int pm_SSM_write(int, char*, int, int, int, int);  /* M15 */

typedef struct {
	char *msgbuf;
	int msglen;
	int msgtype;
	int msgsrc;
	int msgdest;
} SSM_save_t;  /* M19 */

/*---------------------- M19 -------------------------------------

  save a structured socket message

  Parameters:     A0 = pm_SSM_save(A1, A2, A3, A4, A5)

                  A1    message buffer                char *
                  A2    size of message buffer        int 
                  A3    message type                  int
                  A4    message source                int
                  A5    message destination node      int

                  A0    Return Code                   SSM_save_t*
-----------------------------------------------------------------*/

SSM_save_t *pm_SSM_save(char*, int, int, int, int);  /* M19 */

/*-----------------------------------------------------------------

  Receive the header for a Structured Socket Message

  Parameters:     A0 = pm_SSM_read_header(A1, A2, A3, A4, A5)

                  A1    socket to read data from      int
                  A2    length of buffer              int *
                  A3    enumerated message type       int *
                  A4    source node                   int *
                  A5    destination node              int *

                  A0    Return Code                   int
-----------------------------------------------------------------*/

int pm_SSM_read_header ( int sock_id, int *data_buffer_size, int *message_type, int *source_node, int *dest_node  );

/*-----------------------------------------------------------------

  Move a Structured Socket Message

  Parameters:     A0 = pm_SSM_move(A1, A2, A3, A4, A5)

                  A1    destination socket ID         int
                  A2    source socket ID              int
                  A3    number of bytes to move       int M15
                  A4    make buffer flag              int
                  A5    buffer                        char **

                  A0    Return Code                   int
-----------------------------------------------------------------*/

int pm_SSM_move ( int socket_id_out, int socket_id_in, int bytes_to_move, int make_buffer, char **buffer_ptr );  /* M15 */

/*-----------------------------------------------------------------

  Receive the data portion of an SSM

  Parameters:     A0 = pm_SSM_read_data(A1, A2, A3)

                  A1    socket to read data from      int
                  A2    pointer to buffer             char **
                  A3    length of buffer              int

                  A0    Return Code                   int
-----------------------------------------------------------------*/

int pm_SSM_read_data( const int sock_id, char **data_buffer, const int data_buffer_size );

/**************************************************************
  pm structured socket message type declarations
 **************************************************************/

typedef struct
  {
  int ssm_length;                    /* total length including hdr M15 */
  short int ssm_type;                /* enumerated message type    */
  short int ssm_source_node;         /* source node number         */
  short int ssm_dest_node;           /* destination node number    */
  } SSM_HDR_TYPE;

/****************************************************************
   structured message types for the Message Type field of a SSM
 ****************************************************************/

#define SSM_EXITED         1   /* user program exited - report from pmd - data is exit status */
#define SSM_INITIAL_DATA   2   /* initialization data - broadcast to all pmds at startup */
#define SSM_ACK            3   /* acknowledgement - to SIG_REQ, SSM_PARA_NODE_DATA */
#define SSM_SIG_REQ        4   /* signal request - broadcast to all pmds - pmd should send SSM_ACK */
#define SSM_CONTROL        5   /* stdio control and parallel utility - subtypes listed below */
#define SSM_EOF            6   /* end of file on stdin - broadcast to all stdin recipients */
#define SSM_USER_DATA      7   /* user defined data - not used */

#define SSM_PARA_NODE_DATA 9   /* parallel node data - broadcast to all pmds at startup */
#define SSM_INQUIRY        10  /* inquiry from home node to remote - not used */


#define SSM_CMD_STR        13  /* command string - first line is JOBID; second line, 1st token is pgm name */
#define SSM_STRUCT         14  /* mixed character/binary data -- not used */
#define SSM_ACCT_REC       15  /* accounting record - sent by each pmd at command end and pmd exit */
#define SSM_ERROR          16  /* error message generated by POE */
#define SSM_EXIT_REQ       17  /* user program called exit and is waiting - when all have called, send SIGTERM */
#define SSM_PMD_EXIT       18  /* pmd has exited.  Remote socket will close. */
#define SSM_STDIN          19  /* stdin from PM to remote */
#define SSM_STDOUT         20  /* stdout from remote to PM */
#define SSM_STDERR         21  /* stderr from remote to PM */
#define SSM_IO_CLOSED      22  /* user has closed stdio pipe to pmd - subtype is file descriptor number */
#define SSM_MPL_INIT       23  /* MPL node initialization data to home node */
#define SSM_PROFILE_REQ    24  /* Request to profile task - sent by user at task exit */
#define SSM_GO             25  /* Synch message between POE and PMDs */                     
#define SSM_CHILD_PID      26  /* process id of child sending this message */
#define SSM_SIG_NOACK      27  /* signal request - broadcast to all pmds - no ACK from pmd expected */
#define SSM_PROFILE_DONE   28  /* Profiling write finished - sent by user at task exit */
#define SSM_CSS_INIT       29  /* MPL nodes initialization data broadcast to all nodes */
/* M29 removed pdbx definitions */
#define SSM_PULSE          34  /* POE, pmd pulse message, to verify nodes are still up     */
#define SSM_REGISTER       35  /* Remote task data sent to PM (initially used for Attach function ) */
                               /* M18 removed SSM_DBE (36) */
			       /* M27 removed VT SSMs (37 - 43) */
#define SSM_MPCI_CONNECT_DATA   44  /* Connect data for failure analysis M05 */
#define SSM_EXT_DEBUG_ON   45  /* indicates external debugger is on M06 */
#define SSM_EXT_DEBUG_OFF  46  /* indicates external debugger is off M06 */
#define SSM_MY_POE_VERSION 47  /* contains the POE version M07 */
                               /* M18 removed SSM_MY_DBE_VERSION (48) */
#define SSM_CHILD_STOPPED  49  /* indicates child has stopped M09 */
#define SSM_SEND_CHILD_STOPPED 50  /* send child_stopped to parent M09 */
/* M29 removed pdbx debug exit definition */
#define SSM_CHKPT_READY    52  /* Checkpoint Message from task process M11 */
#define SSM_CHKPT_DO       53  /* Checkpoint Message from home node M11 */
#define SSM_CHKPT_DONE     54  /* Checkpoint Message from task process M11 */
#define SSM_CHKPT_COMMIT   55  /* Checkpoint Message from home node M11 */
#define SSM_CHKPT_MAXV     56  /* Restart Message from task process M11 */
#define SSM_CHKPT_MINV     57  /* Restart Message from home node M11 */
#define SSM_CHKPT_NTEXSTV  58  /* Restart Message from task process M11 */
#define SSM_CHKPT_EXSTV    59  /* Restart Message from task process M11 */
#define SSM_CHKPT_RSTR     61  /* Restart Message from home node M11 */
#define SSM_FINALIZE_REQ   62  /* MPI Finalize request from each task M13 */
#define SSM_FINALIZE_ACK   63  /* MPI Finalize acknowledgment from home node M13 */
/* M30 removed third party encryption key token */
/* M29 removed pdbx command string count definition */
				/* M27 removed DCE credentials SSM */
#define SSM_CHKPT_START    67  /* Sent to ckpt thread by PMD */
#define SSM_CHKPT_COMPLETE 68  /* Sent by ckpt thread to PMD */
#define SSM_CHKPT_CONT	   69  /* Sent to main thread by ckpt handler  M19 */
#define SSM_CHKPT_PUSER    70  /* Partial user-initiated ckpt  M19 */
#define SSM_CHKPT_CUSER    71  /* Complete user-initiated ckpt  M19 */
#define SSM_CHKPT_INITIATE 72  /* Ckpt request from PMD  M19 */
#define SSM_RESTART_FAIL   73  /* Restart failed on remote node  M19 */
#define SSM_CHKPT_CANCEL   74  /* PMDs should cancel ckpt operation  M19 */
#define SSM_CMD_RESTART    75  /* PMDs should restart from ckpt file M19 */
#define SSM_CHKPT_EPUSER   76  /* Partial user-initiated ckpt complete  M19 */
#define SSM_CHKPT_ECUSER   77  /* Complete user-initiated ckpt complete M19 */
#define SSM_CHKPT_INTERRUPT 78 /* PMD received a SIGWINCH */
#define SSM_CHKPT_STDIN_SAVE  79
#define SSM_CHKPT_CTLIN_SAVE  80
#define SSM_DBG_CKPT_AWARE      81
#define SSM_DBG_CKPT_START      82
#define SSM_DBG_CKPT_OK         83
#define SSM_DBG_CKPT_PIPECHECK  84
#define SSM_DBG_CKPT_PIPEDATA   85
#define SSM_DBG_CKPT_COMPLETE   86
#define SSM_RESTART_HANDLER_DONE  87
#define SSM_MPI_GLOBAL_INFO  88      /* MPI global task info     M20 */
#define SSM_LAPI_GLOBAL_INFO  89     /* LAPI global task info    M20 */
#define SSM_SEC_CREDS      90  /* CTSec credentials context from home node M21 */
#define SSM_SEC_MECH       91  /* CTSec security mechanism buffer from home node M21 */
#define SSM_SEC_CREDS_FINAL  92  /* Final CTSec credentials context from remote node M22 */
#define SSM_UDP_INFO_REQUEST  93   /* UDP info request          M23 */
#define SSM_UDP_INFO 94            /* UDP info data             M23 */
#define SSM_RESEND 95              /* resend for expected SSM   M23 */
#define SSM_MPI_GLOBAL_INFO_BIN  96     /* Binary MPI global task info  M24 */
#define SSM_LAPI_GLOBAL_INFO_BIN  97    /* Binary LAPI global task info M24 */
#define SSM_PE_INFO_INIT  98   /* HAL initialization info request    M25 */
#define SSM_PE_INFO_REQUEST  99   /* HAL initialization info request    M25 */
#define SSM_PE_INFO  100   /* HAL initialization info request    M25 */
#define SSM_LOCAL_NTBL_INFO   101	/* Local info from PMDs M26 */
#define SSM_GLOBAL_NTBL_INFO  102	/* Global info to all nodes M26 */
#define SSM_GETHOSTNAME       103       /* M28 -- Get real hostname */
#define SSM_RESUME	      104	/* PMD -> task ckpt thread on resume */
#define SSM_RESTARTING	      105	/* PMD -> task ckpt thread on restart */
#define SSM_PE_INFO_REINIT    106       /* M32 MP_SYNC_QP */
 
/****************************************************************
   Structured Socket Message destinations
 ****************************************************************/

#define SSM_HOME_NODE      -1
#define SSM_MY_PMD         -2
/* M27 removed PM Array and VT integrate SSM destinations */
#define SSM_BROADCAST      -5  /* M16 */
#define SSM_MAIN_THREAD	   -6  /* M19 */
#define SSM_CHKPT_THREAD   -7  /* M19 */

/****************************************************************
   SSM return codes and related variables
 ****************************************************************/

#define SSM_OK                          0
#define SSM_SOCK_EOF            -1
#define SSM_IO_ERROR            1
#define SSM_ALLOC_ERROR         2
#define SSM_OTHER_ERROR         3
/* M29 removed pdbx file definitions */

/****************************************************************
   Structured Socket Message SSM_CONTROL subtypes
   The SSM_CONTROL message is only used by the Parallel Utilities

****************************************************************/

#define LOWCMD              0  /* Low range of control subtypes */
#define SSM_SINGLE          0  /* Set Single I/O mode - data is (1) fd, (2) task # */
#define SSM_ORDERED         1  /* Set Ordered I/O mode - data is fd */
#define SSM_UNORDERED       2  /* Set Unordered output mode - data is fd */
#define SSM_ALL             2  /* Set All input mode - data is fd */

#define SSM_MULTI       SSM_ORDERED
#define SSM_ASYNC       SSM_UNORDERED

#define SSM_FLUSH           3  /* Flush I/O according to previous mode - data is fd */
#define SSM_SYNC_REQ        4  /* Request synchronization at this point */
#define SSM_SACK            5  /* Synchronization acknowledgement. */

#define SSM_STOPALL         7  /* Stop all remote tasks (via SIGTERM) - data is exit code */

/* M27 remove misc. PM Array SSMs */

#define SSM_MODE_QUERY     12  /* Used to ask home node what current I/O mode is- data is STDIN or STDOUT */
#define SSM_MODE_RESPONSE  13  /* Used by home node to respond to above - data is current I/O mode */
#define HIGHCMD            13  /* High range of control subtypes */


/******************************************************************************
 Subtypes of SSM_CHKPT_COMMIT, SSM_CHKPT_DONE, SSM_CHKPT_DO, SSM_CHKPT_INITIATE 
******************************************************************************/
#define SSM_CHKPT_CONTINUE  1 /* M19 - Continue task after checkpoint */ 
#define SSM_CHKPT_SUCCESS   2 /* M12 - Checkpoint succeeded */
#define SSM_CHKPT_FAILURE   3 /* M12 - Checkpoint failed */
#define SSM_CHKPT_WAIT      4 /* M19 - Checkpoint must wait for pre-emption */
#define SSM_CHKPT_TERMINATE 5 /* M19 - Terminate task after checkpoint */

#endif
